/*******************************************************************************
                     Computación Paralela 2012 - FaMAF
                      Universidad Nacional de Córdoba

Ejercicio 2 - Grids 2D
======================
El siguiente código pinta una imagen y la muestra en pantalla.
Esto se hace corriendo el kernel block_gradient_2d, que donde cada thread pinta
una celda de un color único al bloque de threads al que pertenece.
A diferencia del ejercicio anterior, este kernel opera con bloques y grillas de
dos dimensiones.

Complete el código agregando las líneas faltantes marcadas con comentarios FALTA.

********************************************************************************/

#include <cuda.h>
#include <cutil.h>
#include <stdio.h>
#include <stdlib.h>

#include "sdlstuff.h"

#define WIDTH 307
#define HEIGHT 233
#define SIZE (WIDTH * HEIGHT)


__global__ void block_gradient_2d(rgba32 * image, size_t width, size_t height) {

    // columna del thread
    size_t x = blockIdx.x * blockDim.x + threadIdx.x;
    // fila del thread
    size_t y = blockIdx.y * blockDim.y + threadIdx.y;
    // indice en el array de memoria
    size_t idx = y * width + x;

    if (x < width && y < height) {
        // color del bloque [0..1] en cada coordenada
        float color_step_x = 256.0f / gridDim.x;
        float color_step_y = 256.0f / gridDim.y;

        rgba32 color;
        color.r = 0u;
        // los bloques son más verdes hacia la derecha
        color.g = color_step_x * blockIdx.x;
        // y más azules hacia abajo
        color.b = color_step_y * blockIdx.y;
        color.a = 255U;
        // escribir color a memoria
        image[idx] = color;
    }
}


int main(int argc, char ** argv) {

    // inicializar sdl
    sdls_init(WIDTH, HEIGHT);

    rgba32 * device_image;
    // FALTA: pedir memoria para la imagen!

    // pintamos la imagen de blanco para ver errores
    CUDA_SAFE_CALL(cudaMemset(device_image, 255, SIZE * sizeof(rgba32)));

    // Correr el kernel
    dim3 block(32, 16); // tamaño del bloque (32x16)
    // FALTA: definir el tamaño del grid!
    // FALTA: llamar al kernel!

    CUT_CHECK_ERROR("Fallo al lanzar el kernel:");

    // esperar que el kernel termine
    CUDA_SAFE_CALL(cudaDeviceSynchronize());

    // pedir memoria del host para la imagen
    rgba32 * host_image;
    host_image = (rgba32 *) malloc(SIZE * sizeof(rgba32));
    // FALTA: copiar imagen al host!

    // dibujar en pantalla
    sdls_drawrectangle(0, 0, WIDTH, HEIGHT, host_image);

    printf("<ENTER> para salir\n");
    getchar();

    // limpieza
    free(host_image);
    CUDA_SAFE_CALL(cudaFree(device_image));

    return 0;
}
